/**
 * -------------------------------------------------------------------------- *
 *
 *  (C) VAISALA OYJ
 *
 *  Upper Air Division
 *
 *  Module name      :  OVL_OverlayManager.asm
 *  Context          :  
 *
 *  Original release :  2003-09-25
 *  Author           :  JKE
 *
 * -------------------------------------------------------------------------- *
 *
 *  Description      :  Overlay Manager
 *
 * -------------------------------------------------------------------------- *
 *  Version control info:
 *
 *  $Date:$
 *
 *  $Revision:$
 *
 *  $Author:$
 * -------------------------------------------------------------------------- *

 */
/*  Revision history:
 *
 *  $Log:$
 *  
 * -------------------------------------------------------------------------- *
*/

// #include files /////////////////////////////////////////////////////////////
#include "def21160.h"
.import "Overlay.h";
.import "def21160.h";

.extern _ov_word_size_run_1;
.extern _ov_word_size_run_2;
.extern _ov_runtimestartaddress_1;
.extern _ov_runtimestartaddress_2;
.extern _ov_startaddress_1;
.extern _ov_startaddress_2;
.extern _ov_word_size_live_1;
.extern _ov_word_size_live_2;
        
.global ov_id_loaded;
.global runWordSize;
.global runAddress;
.global startAddress;
.global liveWordSize;

.global _OverlayManager;

// Global variables ///////////////////////////////////////////////////////////
.section/dm seg_dmda;

.var runWordSize[2] = _ov_word_size_run_1,
                      _ov_word_size_run_2;

.var runAddress[2] = _ov_runtimestartaddress_1,
                     _ov_runtimestartaddress_2;

.var startAddress[2] = _ov_startaddress_1,
                       _ov_startaddress_2;


.var liveWordSize[2] = _ov_word_size_live_1,
                       _ov_word_size_live_2;

/*  Software stack to temporarily store registers corrupted by overlay manager */
.var ov_stack[20];

/* Loaded overlay ID is stored here */
.var ov_id_loaded;
.endseg;


.section/pm             seg_pmco;


/** //////////////////////////////////////////////////////////////////////////]
 // 
 // Description :   Load overlay from external memory and execute wanted 
 //                 function.
 //                 
 //                 If function resides in overlay that is already loaded in 
 //                 the program memory, DMA ain't needed, only function
 //                 is executed then.
 //
 //                 DMA channel 10 is used for transfers. (DMAC10)
 //                 
 // Parameters  :   global      ov_id_loaded    Loaded overlay ID
 //                 .VAR        ov_stack        Stack used to store registers
 //
 //                 These arrays contain linker generated constants needed
 //                 when transferred overlays via DMA.
 //
 //                 global      runWordSize[]       
 //                 global      runAddress[]
 //                 global      startAddress[]
 //                 global      liveWordSize[]
 //
 //                 
 // Return value:   None
 //                 
 /////////////////////////////////////////////////////////////////////////////]
*/

_OverlayManager:

    /*  Store values of registers used by the overlay manager in to the 
        software stack. */
    dm(ov_stack)   = i8;
    dm(ov_stack+1) = m8;
    dm(ov_stack+2) = l8;
    dm(ov_stack+3) = R2;
    dm(ov_stack+7) = R1;

    /* Overlay ID has been defined as R0. R0 is set in the PLIT of LDF. */
    /* Use the overlay id as an index (must subtract one)     */
    R0 = R0 - 1; /*  Overlay ID -1                            */
    m8 = R0;     /*  Offset into the arrays containing linker */
             
    /* Check if overlay is already in the internal memory     */
    R2 = dm(ov_id_loaded);
    R0 = R0 - R2;
    if EQ jump no_dma_required;

    /* Update ov_id_loaded */
    dm(ov_id_loaded) = m8;

    /* Store more values of used registers to stack */
    R0=i0;	dm(ov_stack+4) = R0;
    R0=m0;	dm(ov_stack+5) = R0;
    R0=l0;	dm(ov_stack+6) = R0;

    /* Clear registers */
    l8=0;
    l0=0;

    /* Overlay ID - 1 */
    m0=m8;
    
    /* Save old SYSCON */
    R2 = dm(SYSCON);
    R0 = R2;

    /* HPM in SYSCON must have same packing mode as in DMAC */
    R0 = 0x1c000 ;	
	dm(SYSCON) = R0;

    /*  Get overlay run and live addresses from memory 
        and use to set up the master mode DMA.	            */
    i8 = runAddress;

    i0 = startAddress;

    R0=0;			dm(DMAC10) = R0; /* Disable DMA */

    R0=dm(m0,i0);	
    dm(EI10)=R0;     /* Set external address */
    
    R0=pm(m8,i8);	
    // Address translation
    R1 = 0x40000 ;
    R1 = R0 - R1 ;
    R0 = LSHIFT R1 BY -1 ;
    R1 = R0 + R1 ;
    R0 = 0x40000 ;
    R0 = R0 + R1 ;
    R0 = R0 - 1 ;
    R0 = R0 - 1 ;
    
    dm(II10)=R0;     /* Set internal address */
    i0=liveWordSize;                  /* Number of words stored in internal memory */

    R0=1;			dm(EM10)=R0;     /* External modifier    */

    i8=liveWordSize;                 /* Number of words stored in external memory */
    R0=1;           dm(IM10)=R0;     /* Set internal modifier                     */
	
    R0=dm(m0,i0);	dm(C10)=R0;      /* Set internal count */
    R0=pm(m8,i8);	dm(EC10)=R0;     /* Set external count */

    R0=0x40401;		dm(DMAC10)=R0;   /* Set DMA -control register */
    
    
    /* Wait here until DMA is done */
wait_dma:
	R0 = dm(DMASTAT);
	btst R0 by 10;    /* Test DMA active bit */
	if NOT SZ jump wait_dma;
	
    /* Restore SYSCON */
    dm(SYSCON) = R2;

    /*  Restore register values from stack  */
    l0=dm(ov_stack+6);
    m0=dm(ov_stack+5);
    i0=dm(ov_stack+4);
				
no_dma_required:

    r1=dm(ov_stack+7);
    r2=dm(ov_stack+3);
    l8=dm(ov_stack+2);
    i8=r1;	/* Set i8 to symbol address, this is were PC will jump	*/
    m8=0;	/* to in order to execute the overlay function called.	*/

    r1=dm(ov_stack+1); /* original m8 */
    r0=dm(ov_stack);   /* original i8 */

    /*  Flush the cache.  If an instruction in previous overlay  */
    /*  had been cached, it may be executed instead of the       */
    /*  current overlays instruction.  (If pm transfers align.)  */
    flush cache;

    /*  Jump to the location of the function to be executed.  */
    jump (m8,i8) (db);
    i8=R0;
    m8=r1;
_OverlayManager.end:
.endseg;

/* ----- End of module OVL_OverlayManager.asm ----- */
