/***************************************************************************

        160_prom.asm

        PROM based boot loader of the ADSP21160 processor.

        Copyright (c) 1999 Analog Devices Inc.  All rights reserved.

*****************************************************************************/
 	
/* Warning: After booting process, EP0I bit in IMASK is set. 				*/

/* Define the addresses of various IOP registers							*/

#define SYSCON 0x00
#define WAIT   0x02
#define SYSTAT 0x03	
#define II10   0x40	
#define IM10   0x41	
#define C10    0x42	
#define EI10   0x45	
#define EM10   0x46	
#define EC10   0x47	

#define DMAC10  0x1c	

.SEGMENT/PM     seg_ldr;

/* The loader begins with the interrupts up to and including the low 	*/
/*  priority timer interrupt.											*/

		NOP;NOP;NOP;NOP;        		/* Reserved interrupt         	*/
																		
___lib_RSTI: 	IDLE;					/* Implicit IDLE instruction	*/
				JUMP Start_Loader (DB);	/* Begin loader					*/
				NOP;					/* Pad to next interrupt		*/
				NOP;					/* Pad to next interrupt		*/

___lib_IIOPAI : RTI;
				RTI;
				RTI;
				RTI;

/* Vector for status stack/loop stack overflow or PC stack full:		*/
___lib_SOVFI:   RTI;                                                                        
                RTI;
                RTI;
                RTI;

/* Vector for high priority timer interrupt:							*/
___lib_TMZHI:   RTI;
                RTI;
                RTI;
                RTI;

/* Vector for external interrupts:										*/
___lib_VIRPTI:  RTI;
                RTI;
                RTI;
                RTI;
                
___lib_IRQ2I:   RTI;
                RTI;
                RTI;
                RTI;

___lib_IRQ1I:   RTI;
                RTI;
                RTI;
                RTI;

___lib_IRQ0I:   RTI;
                RTI;
                RTI;
                RTI;

                NOP;NOP;NOP;NOP;		/* Reserved interrupt			*/

/* Vector for Serial port DMA channels:									*/
___lib_SPR0I:   RTI;
                RTI;
                RTI;
                RTI;

___lib_SPR1I:   RTI;
                RTI;
                RTI;
                RTI;

___lib_SPT0I:   RTI;
                RTI;
                RTI;
                RTI;

___lib_SPT1I:   RTI;
                RTI;
                RTI;
                RTI;

/* Vector for link port DMA channels:									*/
___lib_LP0I : 	RTI;
				RTI;
				RTI;
				RTI;

___lib_LP1I : 	RTI;
				RTI;
				RTI;
				RTI;

___lib_LP2I:    RTI;
                RTI;
                RTI;
                RTI;

___lib_LP3I:    RTI;
                RTI;
                RTI;
                RTI;

___lib_LP4I:    RTI;
                RTI;
                RTI;
                RTI;

___lib_LP5I:    RTI;
                RTI;
                RTI;
                RTI;

/* Vectors for External port DMA channels:								*/
___lib_EP0I:	R2=DM(DMAC10);		/* Get DMAC Control setting			*/
				RTI (DB);
				R6=0;
				DM(DMAC10)=R6;		/*  zeroed between uses.			*/

Start_Loader:
/* After power up or reset, default value in SYSCON register is 0x00010020	*/
/* and WAIT register is 0x01ce739c. If for any reason these two value should*/
/* be modified, do it here as following example. Make sure this file does  	*/
/* not exceed 256 words.                                                    */
/*                                                                          */
/*              r0= 0xXXXXXXXX;                                            	*/
/*              DM(SYSCON)= r0;      Modify SYSCON                          */
/*                                                                          */
/*              r0= 0xXXXXXXXX;                                             */
/*              DM(WAIT)= r0;        Modify WAIT                            */
/*                                                                          */
/*                                                                          */

		r0= 0xc000;                                            	
        DM(SYSCON)= r0;

        L0=0;           		/* Zero out L-registers so they	*/
		L4=0;					/*  can be used without wrap	*/
//		L7=0;
		L8=0;
		L12=0;
//		L15=0;

		M5=0;					/* Setup M-registers to use for	*/			
		M6=1;					/*  for various memory transfers*/
		M13=0;
		M14=1;
				
		R10=DM(SYSCON);			/* Read current SYSCON setting	*/
		R12=PASS R10;			/* Hold Initial SYSCON setting	*/
		R11=BSET R10 BY 1;		/* Set BSO bit for reading ROM	*/
		R10=BCLR R10 BY 1;		/* Clear BSO bit for ext write	*/
		

		DM(SYSCON)=R10;			/* Clear BSO bit for writing RAM*/
								
		BIT SET IMASK 0x8000;	/* Enable EP0 interrupt			*/
		BIT SET MODE1 0x1800;	/* Enable interrupts and nesting*/

        R14=0x421;				/* For DMAC10 setting           */

       R15=6;                  	/* EC to load one 48-bit word  	*/
//	   R0=DM(SYSTAT);          	/* Load the Host ID            	*/
//	   R0=FEXT R0 BY 8:3;     

       DM(IM10)=M13;            /* Setup the DMA registers      */
	   DM(EM10)=M14;

       I15 = 0x40004 ;          /* DMA destination address      */

get_addr:
/**    CALL read_PROM_word;
	   COMP(R0, R2);

       IF NE JUMP get_addr; **/
       
       R3 = 0x80062A;

       DM(EI10)=R3;             /* Point to address in PROM     */

read_boot_info: 
	   CALL read_PROM_word;
	   R0=PASS R2;

       CALL read_PROM_word;

load_memory:
	   R0=PASS R0;
	   IF NE JUMP (PC, test_dm16_zero);

/* After the IDLE completes, the following sequence of instructions will*/
/* be executed: (Remember these are in a loop)							*/
/*	R0=R0-R0, DM(I4,M5)=R9, PM(I12,M13)=R11								*/
/* This instructions sets the EQ flag to terminate the loop, writes 	*/
/* the original value to SYSCON, and writes a *new* instruction over	*/
/* itself.  The new instruction is:										*/
/*	PM(0, I8)=PX;														*/
/* This instruction resets the DMA10 vector to whatever it should be.	*/
/* The loop will terminate, because of the previous set EQ.  Instruction*/
/* flow will continue with 0x40005, just like nothing happened!			*/

final_init:	
		R9=0xb16b0000;				/* Load instruction PM(0,I8)=PX;	*/
        R11=BSET R11 BY 9;			/* Set IMDW to 1 for inst write 	*/             

		DM(SYSCON)=R11;				/* Setup to read PROM, inst wrt		*/

		R1=0x040000;				/* Point to destination				*/

		R2=0x100;					/* Load length of last init			*/
		R4=R2*R15 (SSI);			/* Compute external length			*/

		DM(IM10)=M14;				/* Set to increment internal ptr	*/
		I4=0x40004;					/* Point to 0x040004 for patch		*/
		I8=0x40050;					/* Point to DMA10 vector to patch	*/

		I12 = SYSCON;				/* Address of SYSCON				*/

		R4=PASS R4, R11=R12;		/* Clear AZ, hold initial SYSCON	*/

		DO ___lib_RSTI UNTIL EQ;	/* Setup dummy loop					*/

		FLUSH CACHE;				/*R0=PCSTK;Clean off old top-of-loop*/

		R0=0x40004;					/*  and replace with new			*/

		PCSTK=R0;					/*  top-of-loop value				*/

		DM(II10)=R1;				/* Setup DMA to load over ldr		*/
		DM(C10)=R2;					/* Load internal count				*/
		DM(EC10)=R4;				/* Load external count				*/
		JUMP 0x40004 (DB);			/* Jump to start					*/
		DM(DMAC10)=R14;				/* Start DMA transfer				*/

		IDLE;						/* After IDLE, patch then start		*/

test_dm16_zero:	R0=R0-1;
				IF EQ JUMP (PC, dm16_zero);
				R0=R0-1;
				IF NE JUMP (PC, test_dm40_zero);

dm32_zero:
dm16_zero:		R0=R0-R0, I0=R3;
				LCNTR=R2, DO dm16_zero_loop UNTIL LCE;	
dm16_zero_loop:		DM(I0,M6)=R0;

				JUMP read_boot_info;

test_dm40_zero:	R0=R0-1;
				IF NE JUMP (PC, test_dm16_init);

dm40_zero:		PX1=0;	
				PX2=0;
				R0=R0-R0, I0=R3;
				LCNTR=R2, DO dm40_zero_loop UNTIL LCE;
dm40_zero_loop:		DM(I0,M6)=PX;

				JUMP read_boot_info;

test_dm16_init:	R0=R0-1;
				IF NE JUMP (PC, test_dm32_init);

dm16_init:		I0=R3;
				LCNTR=R2, DO dm16_init_loop UNTIL LCE;
					CALL read_PROM_word (DB);
					NOP;             /* NOP's are required for */
					NOP;             /* loop restriction       */

dm16_init_loop:	DM(I0,M6)=R3;

				JUMP read_boot_info;

test_dm32_init:	R0=R0-1;
				IF NE JUMP (PC, test_dm40_init);

dm32_init:		I0=R3;
				R6 = 0x5FFFF;
				comp(R3,R6);
				if gt jump ext_dm32_init;
				
				LCNTR=R2, DO dm32_init_loop UNTIL LCE;
				CALL read_PROM_word(DB);
				NOP;
				NOP;			
dm32_init_loop:	DM(I0,M6)=R3;
				jump fin_dm32_init;
				
ext_dm32_init:	R3 = R3 + 1;
				I0 = R3;
												
				LCNTR=R2, DO ext_dm32_init_loop UNTIL LCE;
				CALL read_PROM_word (DB);
				NOP;	
				NOP;
			
ext_dm32_init_loop:	DM(I0,2)=R3;			
				
fin_dm32_init:
				JUMP read_boot_info;

test_dm40_init:	R0=R0-1;
				IF NE JUMP (PC, test_pm16_zero);

dm40_init:		I0=R3;
				LCNTR=R2, DO dm40_init_loop UNTIL LCE;
					CALL read_PROM_word (DB);
					NOP;
					NOP;

dm40_init_loop:	DM(I0,M6)=PX;
				JUMP read_boot_info;

test_pm16_zero:	R0=R0-1;
				IF EQ JUMP (PC, dm16_zero);
				R0=R0-1;
				IF EQ JUMP (PC, dm32_zero);

test_pm40_zero:	R0=R0-1;
				IF EQ JUMP (PC, dm40_zero);
				R0=R0-1;
				IF NE JUMP (PC, test_pm16_init);

pm48_zero:		PX1=0;
				PX2=0;
				R0=R0-R0, I8=R3;
				LCNTR=R2, DO pm40_zero_loop UNTIL LCE;	
pm40_zero_loop:		PM(I8,M14)=PX;

				JUMP read_boot_info;

test_pm16_init:	R0=R0-1;
				IF EQ JUMP (PC, dm16_init);

test_pm32_init:	R0=R0-1;
				IF EQ JUMP (PC, dm32_init);

test_pm40_init:	R0=R0-1;
				IF EQ JUMP (PC, dm40_init);

test_pm48_init:	R0=R0-1;
				IF NE JUMP test_dm64_zero;

pm48_init:		I8=R3;
				LCNTR=R2, DO pm48_init_loop UNTIL LCE;
					CALL read_PROM_word (DB);
					NOP;
					NOP;

pm48_init_loop:	PM(I8,M14)=PX;
				JUMP read_boot_info;

/*************************************************************/				
				
test_dm64_zero:	R0=R0-1;
				IF NE JUMP (PC, test_dm64_init);

dm64_zero:		PX1=0;	
				PX2=0;
				R0=R0-R0, I0=R3;
				LCNTR=R2, DO dm64_zero_loop UNTIL LCE;
dm64_zero_loop:		DM(I0,M6)=PX (lw);

				JUMP read_boot_info;
				

test_dm64_init:	R0=R0-1;
				IF NE JUMP (PC, test_pm64_zero);

dm64_init:		I0=R3;
				LCNTR=R2, DO dm64_init_loop UNTIL LCE;
					CALL read_PROM_word;		/* read 32 lsbs of 64-bit word	*/
					R10=PX2;					/* save 32 lsbs of 64-bit word	*/
					CALL read_PROM_word;		/* read 32 msbs of 64-bit word	*/
					PX1=R10;					/* restore 32 lsbs to PX1 		*/
					nop;						/* Call cannot be in last 3 instructions of loop	*/
dm64_init_loop:		DM(I0,M6)=PX (lw);			/* write all 64-bits to longword address location	*/
				
				JUMP read_boot_info;
				

test_pm64_zero:	R0=R0-1;
				IF EQ JUMP (PC, dm64_zero);


test_pm64_init:	R0=R0-1;
				IF EQ JUMP (PC, dm64_init);
				
				

/*************************************************************/

read_PROM_word:	R13=DM(SYSCON);				/* Save old value of SYSCON		*/
				DM(SYSCON)=R11;				/* Set BSO bit for ROM read		*/
				DM(II10)=I15;				/* Setup DMA destination address*/
				DM(C10)=M14;				/* Setup DMA internal length	*/
				DM(EC10)=R15;				/* Setup DMA external count		*/
				DM(DMAC10)=R14;				/* Start DMA 					*/
				IDLE;						/* Wait for DMA to complete		*/
				PX=PM(0x40004);				/* Read word from scratch		*/
				DM(SYSCON)=R13;				/* Reset SYSCON to previous		*/
				R2=PX1;						/* Copy PX values into DREGS	*/
				RTS (DB);
				R2=LSHIFT R2 by -16;		/* Downshift 16 bits to adjust for the extension of PX for HH */
				R2=PASS R2, R3=PX2;			/* Test the length				*/

.ENDSEG;





